// filename ******** Main.C ************** 
// Example main program for output compare interrupt example   
// stepper connected to PM3,PM2,PM1,PM0

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing", 
//     Thomsen, copyright (c) 2006,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002,
//    Jonathan W. Valvano 3/17/07

// You may use, edit, run or distribute this file 
//    as long as the above copyright notices remain

#include <hidef.h>      /* common defines and macros */
#include <mc9s12c32.h>     /* derivative information */


#pragma LINK_INFO DERIVATIVE "mc9s12c32"

#include "oc3.h"
#include "pll.h"
const struct State{
  unsigned char Out;      /* Output to Port H */
  const struct State *Next;  
};
typedef const struct State StateType;
typedef StateType *	StatePtr;
StatePtr Pt;  /* Current State */
#define S5  &FSM[0]
#define S6  &FSM[1]
#define S10 &FSM[2]
#define S9  &FSM[3]
StateType FSM[4]={
  {0x05,S6},
  {0x06,S10},
  {0x0A,S9},
  {0x09,S5}
};

//---------------------step---------------------
// move stepper motor one step
// Input: none
// Output: none
// errors there must be a delay between calls
void step(void){
  PTM = (PTM&0xF0)+Pt->Out;
  Pt = Pt->Next;
  PTT ^= 0x01;
}

void main(void) {
  PLL_Init();      // running at 24MHz
  Pt = S5;         // Initial State 
  DDRT |= 0x01;    // PortT bit 0 is an output to an LED
  DDRM = 0x0F;     // stepper connected to PM3,PM2,PM1,PM0
  OC_Start(10,&step);	 // step motor every 10ms
  for(;;) {} /* wait forever */
}